@@ -10,8 +10,68 @@
 
 #include <time.h> // After spidermonkey.h so after Python.h
 
-#include "libjs/jsobj.h"
-#include "libjs/jscntxt.h"
+#include <jsobj.h>
+#include <jscntxt.h>
+
+// Forward decl for add_prop
+JSBool set_prop(JSContext* jscx, JSObject* jsobj, jsval key, jsval* rval);
+
+JSBool
+add_prop(JSContext* jscx, JSObject* jsobj, jsval key, jsval* rval)
+{
+    JSObject* obj = NULL;
+
+    if(JSVAL_IS_NULL(*rval) || !JSVAL_IS_OBJECT(*rval)) return JS_TRUE;
+
+    obj = JSVAL_TO_OBJECT(*rval);
+    if(JS_ObjectIsFunction(jscx, obj)) return set_prop(jscx, jsobj, key, rval);
+    return JS_TRUE;
+}
+
+JSBool
+del_prop(JSContext* jscx, JSObject* jsobj, jsval key, jsval* rval)
+{
+    Context* pycx = NULL;
+    PyObject* pykey = NULL;
+    PyObject* pyval = NULL;
+    JSBool ret = JS_FALSE;
+
+    pycx = (Context*) JS_GetContextPrivate(jscx);
+    if(pycx == NULL)
+    {
+        JS_ReportError(jscx, "Failed to get Python context.");
+        goto done;
+    }
+
+    // Bail if there's no registered global handler.
+    if(pycx->global == NULL)
+    {
+        ret = JS_TRUE;
+        goto done;
+    }
+    
+    // Check access to python land.
+    if(Context_has_access(pycx, jscx, pycx->global, pykey) <= 0) goto done;
+
+    // Bail if the global doesn't have a __delitem__
+    if(!PyObject_HasAttrString(pycx->global, "__delitem__"))
+    {
+        ret = JS_TRUE;
+        goto done;
+    }
+
+    pykey = js2py(pycx, key);
+    if(pykey == NULL) goto done;
+
+    if(PyObject_DelItem(pycx->global, pykey) < 0) goto done;
+
+    ret = JS_TRUE;
+
+done:
+    Py_XDECREF(pykey);
+    Py_XDECREF(pyval);
+    return ret;
+}
 
 JSBool
 get_prop(JSContext* jscx, JSObject* jsobj, jsval key, jsval* rval)
@@ -38,6 +98,8 @@ get_prop(JSContext* jscx, JSObject* jsobj, jsval key, jsval* rval)
     pykey = js2py(pycx, key);
     if(pykey == NULL) goto done;
 
+    if(Context_has_access(pycx, jscx, pycx->global, pykey) <= 0) goto done;
+
     pyval = PyObject_GetItem(pycx->global, pykey);
     if(pyval == NULL)
     {
@@ -84,6 +146,8 @@ set_prop(JSContext* jscx, JSObject* jsobj, jsval key, jsval* rval)
     pykey = js2py(pycx, key);
     if(pykey == NULL) goto done;
 
+    if(Context_has_access(pycx, jscx, pycx->global, pykey) <= 0) goto done;
+
     pyval = js2py(pycx, *rval);
     if(pyval == NULL) goto done;
 
@@ -121,6 +185,8 @@ resolve(JSContext* jscx, JSObject* jsobj, jsval key)
 
     pykey = js2py(pycx, key);
     if(pykey == NULL) goto done;
+    
+    if(Context_has_access(pycx, jscx, pycx->global, pykey) <= 0) goto done;
 
     if(!PyMapping_HasKey(pycx->global, pykey))
     {
@@ -152,8 +218,8 @@ static JSClass
 js_global_class = {
     "JSGlobalObjectClass",
     JSCLASS_GLOBAL_FLAGS,
-    JS_PropertyStub,
-    JS_PropertyStub,
+    add_prop,
+    del_prop,
     get_prop,
     set_prop,
     JS_EnumerateStub,
@@ -221,14 +287,22 @@ Context_new(PyTypeObject* type, PyObject* args, PyObject* kwargs)
     Context* self = NULL;
     Runtime* runtime = NULL;
     PyObject* global = NULL;
+    PyObject* access = NULL;
 
-    if(!PyArg_ParseTuple(
-        args,
-        "O!|O",
+    char* keywords[] = {"runtime", "glbl", "access", NULL};
+
+    if(!PyArg_ParseTupleAndKeywords(
+        args, kwargs,
+        "O!|OO",
+        keywords,
         RuntimeType, &runtime,
-        &global
+        &global,
+        &access
     )) goto error;
 
+    if(global == Py_None) global = NULL;
+    if(access == Py_None) access = NULL;
+
     if(global != NULL && !PyMapping_Check(global))
     {
         PyErr_SetString(PyExc_TypeError,
@@ -236,6 +310,13 @@ Context_new(PyTypeObject* type, PyObject* args, PyObject* kwargs)
         goto error;
     }
 
+    if(access != NULL && !PyCallable_Check(access))
+    {
+        PyErr_SetString(PyExc_TypeError,
+                            "Access handler must be callable.");
+        goto error;
+    }
+
     self = (Context*) type->tp_alloc(type, 0);
     if(self == NULL) goto error;
 
@@ -290,6 +371,9 @@ Context_new(PyTypeObject* type, PyObject* args, PyObject* kwargs)
     if(global != NULL) Py_INCREF(global);
     self->global = global;
 
+    if(access != NULL) Py_INCREF(access);
+    self->access = access;
+
     // Setup counters for resource limits
     self->branch_count = 0;
     self->max_time = 0;
@@ -329,6 +413,7 @@ Context_dealloc(Context* self)
     }
 
     Py_XDECREF(self->global);
+    Py_XDECREF(self->access);
     Py_XDECREF(self->objects);
     Py_XDECREF(self->classes);
     Py_XDECREF(self->rt);
@@ -419,6 +504,42 @@ success:
     return ret;
 }
 
+PyObject*
+Context_set_access(Context* self, PyObject* args, PyObject* kwargs)
+{
+    PyObject* ret = NULL;
+    PyObject* newval = NULL;
+
+    if(!PyArg_ParseTuple(args, "|O", &newval)) goto done;
+    if(newval != NULL && newval != Py_None)
+    {
+        if(!PyCallable_Check(newval))
+        {
+            PyErr_SetString(PyExc_TypeError,
+                                    "Access handler must be callable.");
+            ret = NULL;
+            goto done;
+        }
+    }
+
+    ret = self->access;
+
+    if(newval != NULL && newval != Py_None)
+    {
+        Py_INCREF(newval);
+        self->access = newval;
+    }
+
+    if(ret == NULL)
+    {
+        ret = Py_None;
+        Py_INCREF(ret);
+    }
+
+done:
+    return ret;
+}
+
 PyObject*
 Context_execute(Context* self, PyObject* args, PyObject* kwargs)
 {
@@ -509,13 +630,13 @@ PyObject*
 Context_max_time(Context* self, PyObject* args, PyObject* kwargs)
 {
     PyObject* ret = NULL;
-    time_t curr_max = (time_t) -1;
-    time_t new_max = (time_t) -1;
+    int curr_max = -1;
+    int new_max = -1;
 
-    if(!PyArg_ParseTuple(args, "|I", &new_max)) goto done;
+    if(!PyArg_ParseTuple(args, "|i", &new_max)) goto done;
 
     curr_max = self->max_time;
-    if(new_max != ((time_t) -1)) self->max_time = new_max;
+    if(new_max > 0) self->max_time = (time_t) new_max;
 
     ret = PyLong_FromLong((long) curr_max);
 
@@ -540,6 +661,12 @@ static PyMethodDef Context_methods[] = {
         METH_VARARGS,
         "Remove a global object in the JS VM."
     },
+    {
+        "set_access",
+        (PyCFunction)Context_set_access,
+        METH_VARARGS,
+        "Set the access handler for wrapped python objects."
+    },
     {
         "execute",
         (PyCFunction)Context_execute,
@@ -609,6 +736,42 @@ PyTypeObject _ContextType = {
     Context_new,                                /*tp_new*/
 };
 
+int
+Context_has_access(Context* pycx, JSContext* jscx, PyObject* obj, PyObject* key)
+{
+    PyObject* tpl = NULL;
+    PyObject* tmp = NULL;
+    int res = -1;
+
+    if(pycx->access == NULL)
+    {
+        res = 1;
+        goto done;
+    }
+
+    tpl = Py_BuildValue("(OO)", obj, key);
+    if(tpl == NULL) goto done;
+
+    tmp = PyObject_Call(pycx->access, tpl, NULL);
+    res = PyObject_IsTrue(tmp);
+
+done:
+    Py_XDECREF(tpl);
+    Py_XDECREF(tmp);
+
+    if(res < 0)
+    {
+        PyErr_Clear();
+        JS_ReportError(jscx, "Failed to check python access.");
+    }
+    else if(res == 0)
+    {
+        JS_ReportError(jscx, "Python access prohibited.");
+    }
+
+    return res;
+}
+
 PyObject*
 Context_get_class(Context* cx, const char* key)
 {
