@@ -18,6 +18,7 @@ PyTypeObject* ContextType = NULL;
 PyTypeObject* ObjectType = NULL;
 PyTypeObject* ArrayType = NULL;
 PyTypeObject* FunctionType = NULL;
+PyTypeObject* IteratorType = NULL;
 PyTypeObject* HashCObjType = NULL;
 PyObject* JSError = NULL;
 
@@ -40,6 +41,8 @@ initspidermonkey(void)
     _FunctionType.tp_base = &_ObjectType;
     if(PyType_Ready(&_FunctionType) < 0) return;
 
+    if(PyType_Ready(&_IteratorType) < 0) return;
+
     if(PyType_Ready(&_HashCObjType) < 0) return;
     
     m = Py_InitModule3("spidermonkey", spidermonkey_methods,
@@ -70,6 +73,10 @@ initspidermonkey(void)
     Py_INCREF(FunctionType);
     PyModule_AddObject(m, "Function", (PyObject*) FunctionType);
 
+    IteratorType = &_IteratorType;
+    Py_INCREF(IteratorType);
+    // No module access on purpose.
+
     HashCObjType = &_HashCObjType;
     Py_INCREF(HashCObjType);
     // Don't add access from the module on purpose.
